import plistlib
from pathlib import Path

import log

from . import script

FUNCTIONS = Path(__file__).parent / "browser.applescript"
PREFERENCES = (
    Path.home()
    / "Library"
    / "Preferences"
    / "com.apple.LaunchServices/com.apple.launchservices.secure.plist"
)

NAMES = {
    "com.apple.safari": "Safari",
    "com.google.chrome": "Google Chrome",
    "org.mozilla.firefox": "Firefox",
}
DEFAULT = NAMES["com.google.chrome"]


def detect(data: dict | None = None) -> str:
    if data is None:
        if PREFERENCES.exists():
            with PREFERENCES.open("rb") as fp:
                data = plistlib.load(fp)
        else:
            log.debug(f"File not found: {PREFERENCES}")
            data = {}

    for handler in data.get("LSHandlers", []):
        if handler.get("LSHandlerURLScheme") == "http":
            role = handler["LSHandlerRoleAll"]
            name = NAMES[role]
            log.info(f"Detected default browser: {name}")
            return name

    log.warn("Unable to determine the default browser")
    return DEFAULT


def close(name: str = "") -> bool:
    replacements = {DEFAULT: name or detect()}
    return script.call(FUNCTIONS, "close()", replacements)



from dataclasses import dataclass

import log
from datafiles import datafile, field


@dataclass
class Workspace:
    name: str
    profile: str = ""
    active: bool = True
    uses: int = 0


@datafile("~/Library/Preferences/slackoff.yml")
@dataclass
class Settings:
    workspaces: list[Workspace] = field(default_factory=list)

    def activate(self, name: str, profile: str | None = None):
        log.debug(f"Marking workspace active: {name}")
        workspace = self._update(name, True, profile)
        workspace.uses += 1

    def deactivate(self, name: str):
        log.debug(f"Marking workspace inactive: {name}")
        self._update(name, False)

    def get_profile(self, name: str) -> str | None:
        for workspace in self.workspaces:
            if workspace.name == name and workspace.profile:
                return workspace.profile
        return None

    def _update(self, name: str, active: bool, profile: str | None = None):
        for workspace in self.workspaces:
            if workspace.name == name:
                workspace.active = active
                if profile is not None:
                    workspace.profile = profile
                return workspace

        workspace = Workspace(name, profile or "", active, 0)
        self.workspaces.append(workspace)
        return workspace


settings = Settings()



import sys
import time
from contextlib import suppress

import click
import log
import pync

from . import __version__, browser, slack
from .config import settings


def clean_channel(_ctx, _param, value: str | None) -> str:
    if value is None:
        return ""
    if value := value.strip("# "):
        return value
    raise click.BadParameter("Invalid channel name.")


@click.command(help="Automatically sign out of Slack workspaces.")
@click.argument("workspace", nargs=-1)
@click.option(
    "-i", "--signin", is_flag=True, default=False, help="Only attempt to sign in."
)
@click.option(
    "-p",
    "--profile",
    metavar="NAME",
    default=None,
    help="Browser profile for sign in.",
)
@click.option(
    "-o", "--signout", is_flag=True, default=False, help="Only attempt to sign out."
)
@click.option(
    "-m",
    "--mute",
    metavar="NAME",
    callback=clean_channel,
    help="Mute the specified channel.",
)
@click.option(
    "-u",
    "--unmute",
    metavar="NAME",
    callback=clean_channel,
    help="Unmute the specified channel.",
)
@click.option(
    "--debug", is_flag=True, default=False, help="Show verbose logging output."
)
@click.version_option(__version__)
@click.help_option("-h", "--help")
def main(
    workspace: str,
    signin: bool,
    profile: str | None,
    signout: bool,
    mute: str,
    unmute: str,
    debug: bool,
):
    log.init(debug=debug, format="%(levelname)s: %(message)s")

    explicit = bool(workspace)
    workspace = get_workspace(workspace)

    if not (signin or signout) and not slack.activate():
        sys.exit(1)

    if signin:
        try:
            code = 0 if attempt_signin(workspace, profile) else 2
        except ValueError as err:
            click.echo(err, err=True)
            sys.exit(2)
        sys.exit(code)

    if signout:
        attempt_signout(workspace)
        sys.exit(0)

    if mute:
        code = 0 if attempt_mute(workspace, explicit, mute) else 2
        sys.exit(code)

    if unmute:
        code = 0 if attempt_unmute(workspace, explicit, unmute) else 2
        sys.exit(code)

    if not attempt_signout(workspace):
        click.echo(f"Signing in to {workspace}")
        try:
            attempt_signin(workspace, profile)
        except ValueError as err:
            click.echo(err, err=True)
            sys.exit(2)


def get_workspace(workspace: str) -> str:
    workspace = " ".join(workspace)

    if not workspace:
        if settings.workspaces:
            workspace = settings.workspaces[0].name
        else:
            workspace = click.prompt("Slack workspace")

    log.debug(f"Modifying workspace: {workspace}")
    return workspace


def attempt_signin(workspace: str, profile: str | None = None) -> bool:
    if profile is None:
        profile = settings.get_profile(workspace)
    if profile and browser.detect() != "Google Chrome":
        raise ValueError(
            "--profile is only supported when Chrome is the default browser"
        )
    if not slack.signin(workspace, profile=profile):
        message = f"Click 'Open' to sign in to {workspace}"

        if not slack.ready(workspace):
            pync.notify(message, title="Slackoff")
            click.echo(message + "...")

        with suppress(KeyboardInterrupt):
            while not slack.ready(workspace):
                log.debug(f"Waiting for workspace: {workspace}")
                time.sleep(1)
            click.echo(f"Signed in to of {workspace}")

        browser.close()

    settings.activate(workspace, profile)
    return True


def attempt_signout(workspace) -> bool:
    if slack.signout(workspace):
        click.echo(f"Signed out of {workspace}")
        settings.deactivate(workspace)
        log.debug("Waiting for extra browser tabs")
        time.sleep(10)
        browser.close()
        return True

    click.echo(f"Currently signed out of {workspace}")
    return False


def attempt_mute(workspace: str, explicit: bool, channel: str) -> bool:
    ready = slack.ready(workspace)
    if explicit:
        if not ready:
            ready = attempt_signin(workspace, None)
        if not ready:
            click.echo(f"Workspace not available: {workspace}")
            return False
    elif not ready:
        workspace = "current workspace"

    click.echo(f"Muting #{channel} in {workspace}")
    return slack.mute(workspace, channel)


def attempt_unmute(workspace: str, explicit: bool, channel: str) -> bool:
    ready = slack.ready(workspace)
    if explicit:
        if not ready:
            ready = attempt_signin(workspace, None)
        if not ready:
            click.echo(f"Workspace not available: {workspace}")
            return False
    elif not ready:
        workspace = "current workspace"

    click.echo(f"Unmuting #{channel} in {workspace}")
    return slack.unmute(workspace, channel)


if __name__ == "__main__":  # pragma: no cover
    main()  # pylint: disable=no-value-for-parameter



# pylint: disable=redefined-outer-name,unused-variable,expression-not-assigned,singleton-comparison

from slackoff import browser


def describe_detect():
    def it_returns_browser_name(expect):
        expect(browser.detect()) != ""

    def it_defaults_to_google_chrome(expect):
        expect(browser.detect({})) == "Google Chrome"


def describe_close():
    def it_handles_unknown_browsers(expect):
        expect(browser.close("Foobar")) == False



from importlib.metadata import PackageNotFoundError, version

try:
    __version__ = version("slackoff")
except PackageNotFoundError:
    __version__ = "(local)"

del PackageNotFoundError
del version



from pathlib import Path

from . import script

FUNCTIONS = Path(__file__).parent / "slack.applescript"
CHROME_PROFILES = Path.home() / "Library/Application Support/Google/Chrome"


def activate() -> bool:
    return script.call(FUNCTIONS, "activate()")


def ready(workspace: str) -> bool:
    return script.call(FUNCTIONS, f'ready("{workspace}")', show_error=False)


def signin(workspace: str, *, profile: str | None = None) -> bool:
    profile = profile or ""
    if profile and not (CHROME_PROFILES / profile).is_dir():
        raise ValueError(
            f"Profile {profile!r} not found. "
            "Use the profile directory name (e.g. 'Default', 'Profile 1'). "
            f"Check: {CHROME_PROFILES}"
        )
    return script.call(FUNCTIONS, f'signin("{workspace}", "{profile}")')


def signout(workspace: str) -> bool:
    return script.call(FUNCTIONS, f'signout("{workspace}")', show_error=False)


def mute(workspace: str, channel: str) -> bool:
    return script.call(FUNCTIONS, f'mute("{workspace}", "{channel}")')


def unmute(workspace: str, channel: str) -> bool:
    return script.call(FUNCTIONS, f'unmute("{workspace}", "{channel}")')
